/*
 * n_flash_interface.c
 *
 * Airstar
 * Author: Simon EMBERGER
 * Created: 07/05/2019
 * Functions created for the RF application base on the FDS librairy
 *
 *
 * IMPORTANT : THE FLASH ADDRESS SETTING IS DONE MANUALY IN THE FUNCTION flash_bounds_set 
 *             TO AVOID ERASING THE BOOTLOADER OF THE DONGLE
 *
 * 
*/ 

#include "sdk_config.h"
#include "n_flash_interface.h"

#include "nrf_fstorage.h"
#include "nrf_soc.h"

#include "crc32.h"

#include "string.h"

#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "nrf_log.h"

#include "app_scheduler.h"

#ifdef SOFTDEVICE_PRESENT
#include "nrf_fstorage_sd.h"
#endif


#include "fds.h"


#define FILE_ID       0x0001  /* The ID of the file to write the records into. */
#define RECORD_KEY    0x1111  /* A key for the first record. */
#define FDS_SUCCESS NRF_SUCCESS
 


volatile bool fdsInitialized;
volatile bool writeTerminated;

uint8_t array[12] = {0};

//static 	fds_record_t  netRecord = {	
//								.file_id           = FILE_ID,
//								.key               = RECORD_KEY,
//								.data.p_data       = &array,
//								.data.length_words = (sizeof(array) + 3) /sizeof(uint32_t),   /* one word is four bytes. */
//};


//static fds_record_desc_t   record_desc = {0};





/************************************* fds_evt_handler ***************************************/
/** 
* @Brief Simple event handler to handle errors during initialization. 
*/
static void fds_evt_handler(fds_evt_t const * p_evt)
{
		switch (p_evt->id){
			case FDS_EVT_INIT:
                    NRF_LOG_INFO("FDS_EVT_INIT, result: 0x%02x", p_evt->result);
					if (p_evt->result == FDS_SUCCESS){
							 fdsInitialized = true;
					}
					else{	
						fds_init();	
					}
					break;

			case FDS_EVT_WRITE:
                    NRF_LOG_INFO("FDS_EVT_WRITE, result: 0x%02x", p_evt->result);
					if (p_evt->result == FDS_SUCCESS){
						 writeTerminated = true;
					}
					break;
					
			case FDS_EVT_UPDATE:
                    NRF_LOG_INFO("FDS_EVT_UPDATE, result: 0x%02x", p_evt->result);
					if (p_evt->result == FDS_SUCCESS){
						 writeTerminated = true;
					}
					break;
					
			case FDS_EVT_DEL_RECORD:
                    NRF_LOG_INFO("FDS_EVT_DEL_RECORD, result: 0x%02x", p_evt->result);
					if (p_evt->result == FDS_SUCCESS){    
					}
				break;
			
			 case FDS_EVT_GC:
                    NRF_LOG_INFO("FDS_EVT_GC, result: 0x%02x", p_evt->result);
					if (p_evt->result == FDS_SUCCESS){
					}
					break;
			default:
                    NRF_LOG_INFO("Default FDS event 0x%02x", p_evt->id);
					break;
    }
}


/************************************* power_manage ***************************************/
//static void power_manage(void)
//{
//#ifdef SOFTDEVICE_PRESENT
//    if (NRF_LOG_PROCESS() == false)
//    {
//        //(void) sd_app_evt_wait();
//    }
//#else
//    __WFE();
//#endif
//}



/************************************* wait_for_fds_ready ***************************************/
/**@brief   Wait for fds to initialize. */
static void wait_for_fds_ready(void){
    while (fdsInitialized == false){
        //NRF_LOG_INFO("not ready, %02x", fdsInitialized);
        //power_manage();
        app_sched_execute();
    }
}


/************************************* flash_init ***************************************/
void flash_init(void){
    NRF_LOG_INFO("inside flash_init");
	ret_code_t ret = fds_register(fds_evt_handler);
	if (ret != FDS_SUCCESS){
	}

	ret = fds_init();	
    NRF_LOG_INFO("fds_init ret = 0x%02x", ret);
	APP_ERROR_CHECK(ret);
	
	wait_for_fds_ready();
	writeTerminated = true;
}
